@@ -6,15 +6,16 @@ import itertools
 import ntpath
 import posixpath
 import re
+import warnings
 from pathlib import Path, PurePath
-from typing import Optional, Pattern, Tuple
+from typing import Optional, Pattern, Sequence, Tuple
 
 from ._base import AbstractSanitizer, AbstractValidator, BaseFile, BaseValidator
 from ._common import findall_to_str, to_str, validate_pathtype
 from ._const import DEFAULT_MIN_LEN, INVALID_CHAR_ERR_MSG_TMPL, Platform
 from ._types import PathType, PlatformType
-from .error import ErrorReason, InvalidCharError, ValidationError
-from .handler import NullValueHandler
+from .error import ErrorAttrKey, ErrorReason, InvalidCharError, ValidationError
+from .handler import ReservedNameHandler, ValidationErrorHandler
 
 
 _DEFAULT_MAX_FILENAME_LEN = 255
@@ -30,8 +31,9 @@ class FileNameSanitizer(AbstractSanitizer):
         max_len: int = _DEFAULT_MAX_FILENAME_LEN,
         fs_encoding: Optional[str] = None,
         platform: Optional[PlatformType] = None,
-        check_reserved: bool = True,
-        null_value_handler: Optional[NullValueHandler] = None,
+        null_value_handler: Optional[ValidationErrorHandler] = None,
+        reserved_name_handler: Optional[ValidationErrorHandler] = None,
+        additional_reserved_names: Optional[Sequence[str]] = None,
         validate_after_sanitize: bool = False,
         validator: Optional[AbstractValidator] = None,
     ) -> None:
@@ -42,15 +44,17 @@ class FileNameSanitizer(AbstractSanitizer):
                 min_len=DEFAULT_MIN_LEN,
                 max_len=max_len,
                 fs_encoding=fs_encoding,
-                check_reserved=check_reserved,
+                check_reserved=True,
+                additional_reserved_names=additional_reserved_names,
                 platform=platform,
             )
 
         super().__init__(
             max_len=max_len,
             fs_encoding=fs_encoding,
-            check_reserved=check_reserved,
             null_value_handler=null_value_handler,
+            reserved_name_handler=reserved_name_handler,
+            additional_reserved_names=additional_reserved_names,
             platform_max_len=_DEFAULT_MAX_FILENAME_LEN,
             platform=platform,
             validate_after_sanitize=validate_after_sanitize,
@@ -76,10 +80,12 @@ class FileNameSanitizer(AbstractSanitizer):
         try:
             self._validator.validate(sanitized_filename)
         except ValidationError as e:
-            if e.reason == ErrorReason.RESERVED_NAME and e.reusable_name is False:
-                sanitized_filename = re.sub(
-                    re.escape(e.reserved_name), f"{e.reserved_name}_", sanitized_filename
-                )
+            if e.reason == ErrorReason.RESERVED_NAME:
+                replacement_word = self._reserved_name_handler(e)
+                if e.reserved_name != replacement_word:
+                    sanitized_filename = re.sub(
+                        re.escape(e.reserved_name), replacement_word, sanitized_filename
+                    )
             elif e.reason == ErrorReason.INVALID_CHARACTER and self._is_windows(
                 include_universal=True
             ):
@@ -126,19 +132,19 @@ class FileNameValidator(BaseValidator):
         common_keywords = super().reserved_keywords
 
         if self._is_universal():
-            return (
+            word_set = set(
                 common_keywords
                 + self._WINDOWS_RESERVED_FILE_NAMES
                 + self._MACOS_RESERVED_FILE_NAMES
             )
+        elif self._is_windows():
+            word_set = set(common_keywords + self._WINDOWS_RESERVED_FILE_NAMES)
+        elif self._is_posix() or self._is_macos():
+            word_set = set(common_keywords + self._MACOS_RESERVED_FILE_NAMES)
+        else:
+            word_set = set(common_keywords)
 
-        if self._is_windows():
-            return common_keywords + self._WINDOWS_RESERVED_FILE_NAMES
-
-        if self._is_posix() or self._is_macos():
-            return common_keywords + self._MACOS_RESERVED_FILE_NAMES
-
-        return common_keywords
+        return tuple(sorted(word_set))
 
     def __init__(
         self,
@@ -147,12 +153,14 @@ class FileNameValidator(BaseValidator):
         fs_encoding: Optional[str] = None,
         platform: Optional[PlatformType] = None,
         check_reserved: bool = True,
+        additional_reserved_names: Optional[Sequence[str]] = None,
     ) -> None:
         super().__init__(
             min_len=min_len,
             max_len=max_len,
             fs_encoding=fs_encoding,
             check_reserved=check_reserved,
+            additional_reserved_names=additional_reserved_names,
             platform_max_len=_DEFAULT_MAX_FILENAME_LEN,
             platform=platform,
         )
@@ -166,9 +174,10 @@ class FileNameValidator(BaseValidator):
         self.validate_abspath(unicode_filename)
 
         err_kwargs = {
-            "reason": ErrorReason.INVALID_LENGTH,
-            "platform": self.platform,
-            "fs_encoding": self._fs_encoding,
+            ErrorAttrKey.REASON: ErrorReason.INVALID_LENGTH,
+            ErrorAttrKey.PLATFORM: self.platform,
+            ErrorAttrKey.FS_ENCODING: self._fs_encoding,
+            ErrorAttrKey.BYTE_COUNT: byte_ct,
         }
         if byte_ct > self.max_len:
             raise ValidationError(
@@ -260,6 +269,7 @@ def validate_filename(
     max_len: int = _DEFAULT_MAX_FILENAME_LEN,
     fs_encoding: Optional[str] = None,
     check_reserved: bool = True,
+    additional_reserved_names: Optional[Sequence[str]] = None,
 ) -> None:
     """Verifying whether the ``filename`` is a valid file name or not.
 
@@ -287,6 +297,9 @@ def validate_filename(
             If |None|, get the value from the execution environment.
         check_reserved:
             If |True|, check reserved names of the ``platform``.
+        additional_reserved_names:
+            Additional reserved names to check.
+            Case insensitive.
 
     Raises:
         ValidationError (ErrorReason.INVALID_LENGTH):
@@ -315,6 +328,7 @@ def validate_filename(
         max_len=max_len,
         fs_encoding=fs_encoding,
         check_reserved=check_reserved,
+        additional_reserved_names=additional_reserved_names,
     ).validate(filename)
 
 
@@ -325,12 +339,15 @@ def is_valid_filename(
     max_len: Optional[int] = None,
     fs_encoding: Optional[str] = None,
     check_reserved: bool = True,
+    additional_reserved_names: Optional[Sequence[str]] = None,
 ) -> bool:
     """Check whether the ``filename`` is a valid name or not.
 
     Args:
         filename:
             A filename to be checked.
+        platform:
+            Target platform name of the filename.
 
     Example:
         :ref:`example-is-valid-filename`
@@ -345,6 +362,7 @@ def is_valid_filename(
         max_len=-1 if max_len is None else max_len,
         fs_encoding=fs_encoding,
         check_reserved=check_reserved,
+        additional_reserved_names=additional_reserved_names,
     ).is_valid(filename)
 
 
@@ -354,8 +372,10 @@ def sanitize_filename(
     platform: Optional[PlatformType] = None,
     max_len: Optional[int] = _DEFAULT_MAX_FILENAME_LEN,
     fs_encoding: Optional[str] = None,
-    check_reserved: bool = True,
-    null_value_handler: Optional[NullValueHandler] = None,
+    check_reserved: Optional[bool] = None,
+    null_value_handler: Optional[ValidationErrorHandler] = None,
+    reserved_name_handler: Optional[ValidationErrorHandler] = None,
+    additional_reserved_names: Optional[Sequence[str]] = None,
     validate_after_sanitize: bool = False,
 ) -> PathType:
     """Make a valid filename from a string.
@@ -369,9 +389,8 @@ def sanitize_filename(
             - |invalid_filename_chars|
             - for Windows (or universal) only: |invalid_win_filename_chars|
 
-        - Append underscore (``"_"``) at the tail of the return value if a sanitized name
-          is one of the reserved names by operating systems
-          (only when ``check_reserved`` is |True|).
+        - Replace a value if a sanitized value is a reserved name by operating systems
+          with a specified handler by ``reserved_name_handler``.
 
     Args:
         filename: Filename to sanitize.
@@ -389,11 +408,29 @@ def sanitize_filename(
             Filesystem encoding that used to calculate the byte length of the filename.
             If |None|, get the value from the execution environment.
         check_reserved:
-            If |True|, sanitize reserved names of the ``platform``.
+            [Deprecated] Use 'reserved_name_handler' instead.
         null_value_handler:
             Function called when a value after sanitization is an empty string.
-            Defaults to ``pathvalidate.handler.return_null_string()`` function that
-            return an empty string.
+            You can specify predefined handlers:
+
+                - :py:func:`~.handler.NullValueHandler.return_null_string`
+                - :py:func:`~.handler.NullValueHandler.return_timestamp`
+                - :py:func:`~.handler.raise_error`
+
+            Defaults to :py:func:`.handler.NullValueHandler.return_null_string` that just return ``""``.
+        reserved_name_handler:
+            Function called when a value after sanitization is a reserved name.
+            You can specify predefined handlers:
+
+                - :py:meth:`~.handler.ReservedNameHandler.add_leading_underscore`
+                - :py:meth:`~.handler.ReservedNameHandler.add_trailing_underscore`
+                - :py:meth:`~.handler.ReservedNameHandler.as_is`
+                - :py:func:`~.handler.raise_error`
+
+            Defaults to :py:func:`.handler.add_trailing_underscore`.
+        additional_reserved_names:
+            Additional reserved names to sanitize.
+            Case insensitive.
         validate_after_sanitize:
             Execute validation after sanitization to the file name.
 
@@ -409,11 +446,21 @@ def sanitize_filename(
         :ref:`example-sanitize-filename`
     """
 
+    if check_reserved is not None:
+        warnings.warn(
+            "'check_reserved' is deprecated. Use 'reserved_name_handler' instead.",
+            DeprecationWarning,
+        )
+
+        if check_reserved is False:
+            reserved_name_handler = ReservedNameHandler.as_is
+
     return FileNameSanitizer(
         platform=platform,
         max_len=-1 if max_len is None else max_len,
         fs_encoding=fs_encoding,
-        check_reserved=check_reserved,
         null_value_handler=null_value_handler,
+        reserved_name_handler=reserved_name_handler,
+        additional_reserved_names=additional_reserved_names,
         validate_after_sanitize=validate_after_sanitize,
     ).sanitize(filename, replacement_text)
