@@ -4,9 +4,10 @@
 
 import platform as m_platform
 import random
+import sys
 from collections import OrderedDict
 from itertools import chain, product
-from pathlib import Path
+from pathlib import Path, PurePosixPath, PureWindowsPath
 
 import pytest
 from allpairspy import AllPairs
@@ -19,9 +20,9 @@ from pathvalidate import (
     sanitize_filename,
     validate_filename,
 )
-from pathvalidate._common import is_pathlike_obj, unprintable_ascii_chars
+from pathvalidate._common import unprintable_ascii_chars
 from pathvalidate._filename import FileNameSanitizer, FileNameValidator
-from pathvalidate.handler import raise_error, return_null_string, return_timestamp
+from pathvalidate.handler import NullValueHandler, ReservedNameHandler, raise_error
 
 from ._common import (
     INVALID_FILENAME_CHARS,
@@ -48,7 +49,11 @@ VALID_MULTIBYTE_NAMES = ["新しいテキスト ドキュメント.txt", "新規
 class Test_FileSanitizer:
     @pytest.mark.parametrize(
         ["test_platform", "expected"],
-        [["windows", Platform.WINDOWS], ["linux", Platform.LINUX], ["macos", Platform.MACOS]],
+        [
+            ["windows", Platform.WINDOWS],
+            ["linux", Platform.LINUX],
+            ["macos", Platform.MACOS],
+        ],
     )
     def test_normal_platform_auto(self, monkeypatch, test_platform, expected):
         if test_platform == "windows":
@@ -64,17 +69,20 @@ class Test_FileSanitizer:
 
         assert FileNameSanitizer(255, platform="auto").platform == expected
 
+    def test_normal_additional_reserved_names(self):
+        sanitizer = FileNameSanitizer(additional_reserved_names=["abc"])
+        assert sanitizer.reserved_keywords == ("ABC",)
+
+
+class Test_FileNameValidator:
     @pytest.mark.parametrize(
         ["test_platform", "expected"],
         [
             [
                 "windows",
                 (
-                    "CON",
-                    "PRN",
                     "AUX",
                     "CLOCK$",
-                    "NUL",
                     "COM1",
                     "COM2",
                     "COM3",
@@ -84,6 +92,7 @@ class Test_FileSanitizer:
                     "COM7",
                     "COM8",
                     "COM9",
+                    "CON",
                     "LPT1",
                     "LPT2",
                     "LPT3",
@@ -93,6 +102,8 @@ class Test_FileSanitizer:
                     "LPT7",
                     "LPT8",
                     "LPT9",
+                    "NUL",
+                    "PRN",
                 ),
             ],
             ["linux", ()],
@@ -102,6 +113,16 @@ class Test_FileSanitizer:
     def test_normal_reserved_keywords(self, test_platform, expected):
         assert FileNameValidator(255, platform=test_platform).reserved_keywords == expected
 
+    def test_normal_additional_reserved_names(self):
+        sanitizer = FileNameValidator(additional_reserved_names=["abc", "efg.txt"])
+        assert "ABC" in sanitizer.reserved_keywords
+        assert "EFG.TXT" in sanitizer.reserved_keywords
+
+        sanitizer = FileNameValidator(platform="windows", additional_reserved_names=["CON"])
+        assert (
+            sanitizer.reserved_keywords == FileNameValidator(platform="windows").reserved_keywords
+        )
+
 
 class Test_validate_filename:
     VALID_CHARS = VALID_FILENAME_CHARS
@@ -131,7 +152,11 @@ class Test_validate_filename:
 
     @pytest.mark.parametrize(
         ["platform"],
-        [["linux"], ["macos"], ["posix"]],
+        [
+            ["linux"],
+            ["macos"],
+            ["posix"],
+        ],
     )
     def test_normal_only_whitespaces(self, platform):
         value = "  "
@@ -140,7 +165,10 @@ class Test_validate_filename:
 
     @pytest.mark.parametrize(
         ["platform"],
-        [["windows"], ["universal"]],
+        [
+            ["windows"],
+            ["universal"],
+        ],
     )
     def test_abnormal_only_whitespaces(self, platform):
         value = "  "
@@ -174,10 +202,13 @@ class Test_validate_filename:
         if expected is None:
             validate_filename(value, min_len=min_len)
             assert is_valid_filename(value, min_len=min_len)
-        else:
-            with pytest.raises(ValidationError) as e:
-                validate_filename(value, min_len=min_len)
-            assert e.value.reason == expected
+            return
+
+        with pytest.raises(ValidationError) as e:
+            validate_filename(value, min_len=min_len)
+        assert e.value.reason == expected
+        assert e.value.fs_encoding
+        assert e.value.byte_count > 0
 
     @pytest.mark.parametrize(
         ["value", "platform", "max_len", "expected"],
@@ -198,6 +229,35 @@ class Test_validate_filename:
         with pytest.raises(ValidationError) as e:
             validate_filename(value, platform=platform, max_len=max_len)
         assert e.value.reason == expected
+        assert e.value.fs_encoding
+        assert e.value.byte_count > 0
+
+    @pytest.mark.parametrize(
+        ["value", "platform", "fs_encoding", "max_len", "expected"],
+        [
+            ["あ" * 85, "universal", "utf-8", 255, None],
+            ["あ" * 86, "universal", "utf-8", 255, ErrorReason.INVALID_LENGTH],
+            ["あ" * 126, "universal", "utf-16", 255, None],
+            ["あ" * 127, "universal", "utf-16", 255, ErrorReason.INVALID_LENGTH],
+        ],
+    )
+    def test_max_len_fs_encoding(self, value, platform, fs_encoding, max_len, expected):
+        kwargs = {
+            "platform": platform,
+            "max_len": max_len,
+            "fs_encoding": fs_encoding,
+        }
+
+        if expected is None:
+            validate_filename(value, **kwargs)
+            assert is_valid_filename(value, **kwargs)
+            return
+
+        with pytest.raises(ValidationError) as e:
+            validate_filename(value, **kwargs)
+        assert e.value.reason == expected
+        assert e.value.fs_encoding
+        assert e.value.byte_count > 0
 
     @pytest.mark.parametrize(
         ["value", "min_len", "max_len", "expected"],
@@ -213,9 +273,10 @@ class Test_validate_filename:
         if expected is None:
             validate_filename(value, min_len=min_len, max_len=max_len)
             assert is_valid_filename(value, min_len=min_len, max_len=max_len)
-        else:
-            with pytest.raises(expected):
-                validate_filename(value, min_len=min_len, max_len=max_len)
+            return
+
+        with pytest.raises(expected):
+            validate_filename(value, min_len=min_len, max_len=max_len)
 
     @pytest.mark.skipif(not is_faker_installed(), reason="requires faker")
     @pytest.mark.parametrize(["locale"], [[None], ["ja_JP"]])
@@ -249,6 +310,21 @@ class Test_validate_filename:
         assert e.value.reason == ErrorReason.INVALID_CHARACTER
         assert not is_valid_filename(value, platform=platform)
 
+    @pytest.mark.parametrize(
+        ["value", "platform"],
+        [
+            ["a/b", Platform.UNIVERSAL],
+            ["a*b", Platform.WINDOWS],
+            [PurePosixPath("a/b"), Platform.UNIVERSAL],
+            [PureWindowsPath("a/b"), Platform.WINDOWS],
+        ],
+    )
+    def test_exception_invalid_char_specific_target_platform(self, value, platform):
+        with pytest.raises(ValidationError) as e:
+            validate_filename(value, platform)
+        assert e.value.reason == ErrorReason.INVALID_CHARACTER
+        assert e.value.platform == platform
+
     @pytest.mark.parametrize(
         ["value", "platform"],
         [
@@ -302,6 +378,19 @@ class Test_validate_filename:
 
             assert not is_valid_filename(value, platform=platform)
 
+    @pytest.mark.parametrize(
+        ["value", "arn", "expected"],
+        [
+            ["abc", [], True],
+            ["abc", ["abc"], False],
+            ["Abc", ["abc"], False],
+            ["ABC", ["abc"], False],
+            ["abc.txt", ["abc.txt"], False],
+        ],
+    )
+    def test_normal_additional_reserved_names(self, value, arn, expected):
+        assert is_valid_filename(value, additional_reserved_names=arn) == expected
+
     @pytest.mark.parametrize(
         ["platform", "value", "expected"],
         [
@@ -355,8 +444,8 @@ class Test_validate_filename:
 
         assert e.value.reason == ErrorReason.INVALID_CHARACTER
         assert str(e.value) == (
-            r"invalid char found: invalids=('\r'), value='asdf\rsdf', "
-            "reason=INVALID_CHARACTER, target-platform=Windows"
+            r"[PV1100] invalid characters found: invalids=('\r'), value='asdf\rsdf', "
+            "platform=Windows"
         )  # noqa
 
     @pytest.mark.parametrize(
@@ -409,7 +498,14 @@ class Test_sanitize_filename:
             for pair in AllPairs(
                 OrderedDict(
                     {
-                        "platform": ["posix", "linux", "macos"],
+                        "platform": [
+                            "posix",
+                            "linux",
+                            "macos",
+                            Platform.POSIX,
+                            Platform.LINUX,
+                            Platform.MACOS,
+                        ],
                         "c": INVALID_PATH_CHARS + unprintable_ascii_chars,
                         "repl": REPLACE_TEXT_LIST,
                     }
@@ -421,7 +517,14 @@ class Test_sanitize_filename:
             for pair in AllPairs(
                 OrderedDict(
                     {
-                        "platform": ["posix", "linux", "macos"],
+                        "platform": [
+                            "posix",
+                            "linux",
+                            "macos",
+                            Platform.POSIX,
+                            Platform.LINUX,
+                            Platform.MACOS,
+                        ],
                         "c": [":", "*", "?", '"', "<", ">", "|"],
                         "repl": REPLACE_TEXT_LIST,
                     }
@@ -450,7 +553,7 @@ class Test_sanitize_filename:
     def test_normal_pathlike(self, value, replace_text, expected):
         sanitized_name = sanitize_filename(value, replace_text)
         assert sanitized_name == expected
-        assert is_pathlike_obj(sanitized_name)
+        assert isinstance(sanitized_name, Path)
 
         validate_filename(sanitized_name)
         assert is_valid_filename(sanitized_name)
@@ -466,8 +569,10 @@ class Test_sanitize_filename:
         ],
     )
     def test_normal_null_value_handler(self, value):
-        assert sanitize_filename(value, null_value_handler=return_null_string) == ""
-        assert sanitize_filename(value, null_value_handler=return_timestamp) != ""
+        assert (
+            sanitize_filename(value, null_value_handler=NullValueHandler.return_null_string) == ""
+        )
+        assert sanitize_filename(value, null_value_handler=NullValueHandler.return_timestamp) != ""
         with pytest.raises(ValidationError):
             sanitize_filename(value, null_value_handler=raise_error)
 
@@ -524,6 +629,47 @@ class Test_sanitize_filename:
         assert filename == expected
         assert is_valid_filename(filename, platform=test_platform)
 
+    @pytest.mark.parametrize(
+        ["value", "reserved_name_handler", "expected"],
+        [
+            ["CON", ReservedNameHandler.add_trailing_underscore, "CON_"],
+            ["CON", ReservedNameHandler.add_leading_underscore, "_CON"],
+            ["CON", ReservedNameHandler.as_is, "CON"],
+        ],
+    )
+    def test_normal_reserved_name_handler(self, value, reserved_name_handler, expected):
+        for platform in ["windows", "universal"]:
+            assert (
+                sanitize_filename(
+                    value, platform=platform, reserved_name_handler=reserved_name_handler
+                )
+                == expected
+            )
+
+    def test_exception_reserved_name_handler(self):
+        for platform in ["windows", "universal"]:
+            with pytest.raises(ValidationError) as e:
+                sanitize_filename("CON", platform=platform, reserved_name_handler=raise_error)
+            assert e.value.reason == ErrorReason.RESERVED_NAME
+
+    @pytest.mark.parametrize(
+        ["value", "arn", "expected"],
+        [
+            ["abc", [], "abc"],
+            ["abc", ["abc"], "abc_"],
+        ],
+    )
+    def test_normal_additional_reserved_names(self, value, arn, expected):
+        for platform in ["windows", "universal"]:
+            assert (
+                sanitize_filename(
+                    value,
+                    platform=platform,
+                    additional_reserved_names=arn,
+                )
+                == expected
+            )
+
     @pytest.mark.parametrize(
         ["value", "check_reserved", "expected"],
         [
@@ -532,23 +678,36 @@ class Test_sanitize_filename:
         ],
     )
     def test_normal_check_reserved(self, value, check_reserved, expected):
-        assert (
-            sanitize_filename(value, platform="windows", check_reserved=check_reserved) == expected
-        )
+        for platform in ["windows", "universal"]:
+            assert (
+                sanitize_filename(value, platform=platform, check_reserved=check_reserved)
+                == expected
+            )
 
     @pytest.mark.parametrize(
         ["platform", "value", "expected"],
         [
             ["windows", "period.", "period"],
             ["windows", "space ", "space"],
+            ["windows", " space ", "space"],
             ["windows", "space_and_period .", "space_and_period"],
             ["windows", "space_and_period. ", "space_and_period"],
+            ["windows", " .space_and_period", ".space_and_period"],
+            ["windows", ". space_and_period", ". space_and_period"],
+            ["windows", ". ", "."],
+            ["windows", " .", "."],
+            ["windows", " . ", "."],
+            ["windows", ".. ", ".."],
             ["linux", "period.", "period."],
             ["linux", "space ", "space "],
             ["linux", "space_and_period. ", "space_and_period. "],
             ["universal", "period.", "period"],
             ["universal", "space ", "space"],
             ["universal", "space_and_period .", "space_and_period"],
+            ["universal", ". ", "."],
+            ["universal", " .", "."],
+            ["universal", " . ", "."],
+            ["universal", ".. ", ".."],
         ],
     )
     def test_normal_space_or_period_at_tail(self, platform, value, expected):
@@ -556,7 +715,38 @@ class Test_sanitize_filename:
         assert filename == expected
         assert is_valid_filename(filename, platform=platform)
 
-    @pytest.mark.parametrize(["value", "expected"], [[1, TypeError], [True, TypeError]])
+    @pytest.mark.parametrize(
+        ["platform", "value"],
+        [
+            [platform, value]
+            for platform, value in product(
+                ["windows", "universal"],
+                [
+                    "\a\r",
+                ],
+            )
+        ],
+    )
+    def test_exception_invalid_after_sanitize(self, platform, value):
+        kwargs = {
+            "platform": platform,
+            "replacement_text": "",
+            "validate_after_sanitize": False,
+        }
+        print(f"'{sanitize_filename(value, **kwargs)}'", file=sys.stderr)
+        kwargs["validate_after_sanitize"] = True
+
+        with pytest.raises(ValidationError) as e:
+            sanitize_filename(value, **kwargs)
+        assert e.value.reason == ErrorReason.INVALID_AFTER_SANITIZE
+
+    @pytest.mark.parametrize(
+        ["value", "expected"],
+        [
+            [1, TypeError],
+            [True, TypeError],
+        ],
+    )
     def test_exception_type(self, value, expected):
         with pytest.raises(expected):
             sanitize_filename(value)
