@@ -22,7 +22,7 @@ from pathvalidate import (
 )
 from pathvalidate._common import unprintable_ascii_chars
 from pathvalidate._filepath import FilePathSanitizer, FilePathValidator
-from pathvalidate.handler import raise_error, return_null_string, return_timestamp
+from pathvalidate.handler import NullValueHandler, ReservedNameHandler, raise_error
 
 from ._common import (
     INVALID_PATH_CHARS,
@@ -64,6 +64,10 @@ class Test_FileSanitizer:
 
         assert FilePathSanitizer(255, platform="auto").platform == expected
 
+    def test_normal_additional_reserved_names(self):
+        sanitizer = FilePathSanitizer(additional_reserved_names=["abc"])
+        assert sanitizer.reserved_keywords == ("ABC",)
+
 
 class Test_FilePathValidator:
     @pytest.mark.parametrize(
@@ -78,6 +82,10 @@ class Test_FilePathValidator:
     def test_normal_reserved_keywords(self, test_platform, expected):
         assert FilePathValidator(255, platform=test_platform).reserved_keywords == expected
 
+    def test_normal_additional_reserved_names(self):
+        sanitizer = FilePathValidator(additional_reserved_names=["abc"])
+        assert "ABC" in sanitizer.reserved_keywords
+
 
 class Test_validate_filepath:
     VALID_CHARS = VALID_PATH_CHARS
@@ -176,6 +184,8 @@ class Test_validate_filepath:
         with pytest.raises(ValidationError) as e:
             validate_filepath(value, min_len=min_len)
         assert e.value.reason == expected
+        assert e.value.fs_encoding
+        assert e.value.byte_count > 0
 
     @pytest.mark.parametrize(
         ["value", "platform", "max_len", "expected"],
@@ -211,6 +221,8 @@ class Test_validate_filepath:
         with pytest.raises(ValidationError) as e:
             validate_filepath(value, **kwargs)
         assert e.value.reason == ErrorReason.INVALID_LENGTH
+        assert e.value.fs_encoding
+        assert e.value.byte_count > 0
 
     @pytest.mark.parametrize(
         ["value", "platform", "fs_encoding", "max_len", "expected"],
@@ -236,6 +248,8 @@ class Test_validate_filepath:
         with pytest.raises(ValidationError) as e:
             validate_filepath(value, **kwargs)
         assert e.value.reason == expected
+        assert e.value.fs_encoding
+        assert e.value.byte_count > 0
 
     @pytest.mark.parametrize(
         ["value", "min_len", "max_len", "expected"],
@@ -459,6 +473,16 @@ class Test_validate_filepath:
 
         assert not is_valid_filepath(value, platform=platform)
 
+    @pytest.mark.parametrize(
+        ["value", "arn", "expected"],
+        [
+            ["abc/efg.txt", ["abc"], False],
+            ["abc/efg.txt", ["efg.txt"], False],
+        ],
+    )
+    def test_normal_additional_reserved_names(self, value, arn, expected):
+        assert is_valid_filepath(value, additional_reserved_names=arn) == expected
+
     @pytest.mark.parametrize(
         ["value", "platform", "expected"],
         [
@@ -474,7 +498,7 @@ class Test_validate_filepath:
         assert e.value.reason == expected
         assert str(e.value) == (
             r"[PV1100] invalid characters found: invalids=('\r'), value='asdf\rsdf', "
-            "target-platform=Windows"
+            "platform=Windows"
         )  # noqa
 
     @pytest.mark.parametrize(
@@ -648,6 +672,28 @@ class Test_sanitize_filepath:
         assert filename == expected
         assert is_valid_filepath(filename, platform=test_platform)
 
+    @pytest.mark.parametrize(
+        ["value", "reserved_name_handler", "expected"],
+        [
+            ["CON", ReservedNameHandler.add_trailing_underscore, "CON_"],
+            ["CON", ReservedNameHandler.add_leading_underscore, "_CON"],
+            ["CON", ReservedNameHandler.as_is, "CON"],
+        ],
+    )
+    def test_normal_reserved_name_handler(self, value, reserved_name_handler, expected):
+        assert (
+            sanitize_filepath(
+                value, platform="windows", reserved_name_handler=reserved_name_handler
+            )
+            == expected
+        )
+
+    def test_exception_reserved_name_handler(self):
+        for platform in ["windows", "universal"]:
+            with pytest.raises(ValidationError) as e:
+                sanitize_filepath("CON", platform=platform, reserved_name_handler=raise_error)
+            assert e.value.reason == ErrorReason.RESERVED_NAME
+
     @pytest.mark.parametrize(
         ["value", "check_reserved", "expected"],
         [
@@ -660,6 +706,23 @@ class Test_sanitize_filepath:
             sanitize_filepath(value, platform="windows", check_reserved=check_reserved) == expected
         )
 
+    @pytest.mark.parametrize(
+        ["value", "arn", "expected"],
+        [
+            ["abc", ["abc"], "abc_"],
+        ],
+    )
+    def test_normal_additional_reserved_names(self, value, arn, expected):
+        for platform in ["windows", "universal"]:
+            assert (
+                sanitize_filepath(
+                    value,
+                    platform=platform,
+                    additional_reserved_names=arn,
+                )
+                == expected
+            )
+
     @pytest.mark.parametrize(
         ["value", "replace_text", "expected"],
         [
@@ -730,8 +793,10 @@ class Test_sanitize_filepath:
         ],
     )
     def test_normal_null_value_handler(self, value):
-        assert sanitize_filepath(value, null_value_handler=return_null_string) == ""
-        assert sanitize_filepath(value, null_value_handler=return_timestamp) != ""
+        assert (
+            sanitize_filepath(value, null_value_handler=NullValueHandler.return_null_string) == ""
+        )
+        assert sanitize_filepath(value, null_value_handler=NullValueHandler.return_timestamp) != ""
         with pytest.raises(ValidationError):
             sanitize_filepath(value, null_value_handler=raise_error)
 
